/********************************************************
Copyright (C) 2011, Kengo NAGASHIMA.

This program is free software: you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation, either
version 3 of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public
License along with this program.
If not, see <http://www.gnu.org/licenses/>
********************************************************/

/********************************************************
  Macro name: %kmdata()
  Author: Kengo NAGASHIMA and Yasunori SATO
  Version: 2.2.2
  Last Updated Date: 10/19/2011
  System Recommends:
      Windows XP 32 bit, SAS 9.2,
      set up SAS output fint size = 12pt
  System Requirements:
      Windows XP / Vista / 7, SAS 9.1.3 or later,
      base SAS, SAS/STAT, SAS/GRAPH
********************************************************/

/********************************************************
  Make graph datasets for extended Kaplan-Meier plots

    data: a input dataset name
    time: a survival time variable name
    stra: a stratum variable name
    censor: a censor variable name
    censorv: censored variable code
    out: output graph dataset name
    anno: output annotate dataset name
    afont: annotate text font
    failure: 0 = a plot of estimated survival function
             1 = a plot of estimated 1 - survival function
    adjbase: 0 = none
             1 = baseline adjustment with covariates
                 see the help PhrEG procedure baseline statement
    adjbaseData: a dataset for baseline adjustment
    covariates: covariates for adjustment
    classCovariates: covariates for adjustment to class statement
    ci: 0 = none
        1 = show confidence interval
    pattern: confidence interval filling pattern
             see [Annotate Dictionary STYLE Variable]
    censEXT: 0 = none
             1 = censor extension (needle)
    cHeight: extended censor height
    cWidth: extended censor width
    size: font size of at risk, test and hazard ratio
    step: at risk, mst & hr interval step size
    atrisk: 0 = none
            1 = no. at risk
    atriskOrder: no. at risk order
    atriskYsys: for an outside no. at risk table;
                see the SAS help "YSYS variable."
    atriskPos: for an outside no. at risk table;
               see the SAS help "Position variable."
    atriskStep: for an outside no. at risk table; step size.
    base: at risk base position
    label: at risk label
    test: 0 = none
          1 = show homogeneity test
    testX: = x position (% of data area)
    testY: = y position (% of data area)
    type: logrank = log-rank test
          wilcoxon = generalized wilxocon test
          likelihoodratio = likelihood ratio test
    hr: 0 = none
            1 = show hazard ratio
    hrX: x position (% of data area)
    hrY: y position (% of data area)
    mst: 0 = none
         1 = show median survival time
    mLabX: x position of strata label (% of data area)
    mMedX: x position of median (% of data area)
    mCIX: x position of confidence interval (% of data area)
    mstY: y position (% of data area)

********************************************************/
%macro km_data(
  data, time, stra, censor, censorv,
  out = graph, anno = anno,
  afont = "'Times New Roman'",

  failure = 0,

  adjbase = 0, adjbaseData = , covariates = ,
  classCovariates = ,

  ci = 0, pattern = x4,

  censEXT = 0, cHeight = 0.02, cWidth = 5,

  size = 2.5, step = 5,

  atrisk = 0, atriskOrder = , atriskYsys = 1,
  atriskPos = 3, atriskStep = 5, base = 0, label = "No. at risk",

  test = 0, testX = 90, testY = 100, type = logrank,

  hr = 0, hrX = 90, hrY = 95,

  mst = 0, mLabX = 60, mMedX = 80, mCIX = 98, mstY = 95
);

%if "&atrisk." = "1" & "&atriskorder." = "" %then %do;
  data errorlog;
    error = "The argument 'atriskorder' is needed.";
  proc print noobs; run;
  proc datasets lib = work; delete errorlog; quit;
  %return;
%end;

%if "&adjbase." = "1" %then %do;
  %let censEXT = 0;
  %let atrisk = 0;
  %let test = 0;
  %let hr = 0;
  %let mst = 0;
%end;

libname kmdata "";

data kmdata.data;
  set &data.;
  rename &time. = time &stra. = strata &censor. = censor;
%if "&stra." = "" %then %do;
  data kmdata.data;
    set kmdata.data;
    strata = 1;
%end;
proc sort data = kmdata.data; by strata time;

ods results off; ods exclude all;
proc lifetest data = kmdata.data outsurv = &out.;
  time time * censor(&censorv.);
  strata strata;
  ods output
    HomTests = kmdata.test
    CensoredSummary = kmdata.cs(where=(control_var^="-"))
    ProductLimitEstimates = kmdata.ple Quartiles = kmdata.qest;
run;
%if "&adjbase." = "1" %then %do;
  %if "&sysver." = "9.2" %then proc phreg;
  %else %if "&sysver." = "9.1" %then proc tphreg;
  data = kmdata.data;
    class strata &classCovariates. / param = ref ref = first;
    model time * censor(&censorv.) = strata &covariates. / ties = exact;
    baseline out = &out. covariates = &adjbaseData. survival = _ALL_ / nomean;
  run;
  ods select all; ods results on;
  proc sort data = &out.; by strata;
  data &out.;
    set &out.; by strata;
    retain STRATUM 1;
    _CENSOR_ = 0;
    if SURVIVAL ^= . then output;
    if last.strata then STRATUM = STRATUM + 1;
    rename LowerSurvival = SDF_LCL UpperSurvival = SDF_UCL;
%end;
%else %do;
  %if "&sysver." = "9.2" %then proc phreg;
  %else %if "&sysver." = "9.1" %then proc tphreg;
  data = kmdata.data;
    class strata / param = ref ref = first;
    model time * censor(&censorv.) = strata / rl ties = exact;
    ods output ParameterEstimates = kmdata.hre;
  run;
%end;
ods select all; ods results on;

proc sort data = &out.; by strata;
data &out.;
  set &out.; retain CSDF 0;
  %if "&failure." = "1" %then %do;
    SURVIVAL = 1 - SURVIVAL;
    SDF_LCL  = 1 - SDF_LCL;
    SDF_UCL  = 1 - SDF_UCL;
  %end;
  if Survival = . then Survival = CSDF;
  else CSDF = Survival;
  drop strata;
data &out.;
  set
    &out.(where=(_CENSOR_ ^= 1) drop=CSDF)
    &out.(where=(_CENSOR_ = 1));
  drop _CENSOR_;
data _null_;
  set kmdata.cs;
  call symput("straN", cat(_N_));
proc sort data = &out.; by time;
data &out.;
  merge
  %do i = 1 %to &straN.;
    &out.(where  = (STRATUM=&i.)
          rename = (Survival=SV&i. SDF_LCL=SL&i. SDF_UCL=SU&i.)
          drop   = CSDF)
    &out.(where  = (STRATUM=&i. & CSDF&i.^=.)
          rename = (CSDF=CSDF&i. SDF_LCL=SL&i. SDF_UCL=SU&i.)
          drop   = Survival)
  %end; ;
  by time; drop STRATUM;
proc sort data = &out.; by time;

/* Confidence interval */
%if "&ci." = "1" %then
  %km_ci(&out., time, &straN., &pattern., kmdata, kmdata.ci);
/* Censor extension */
%if "&censEXT." = "1" %then
  %km_censEXT(&out., time, &straN., &cHeight., &cWidth., kmdata.cens);
/* No. at risk */
%if "&atrisk." = "1" %then
  %km_atrisk(kmdata.ple, strata, time, &straN., &atriskorder., &base.,
    &step., &size., &label., &afont., &atriskYsys., &atriskPos.,
    &atriskStep., kmdata, kmdata.atrisk);
/* Homogeneity test */
%if "&Test." = "1" %then
  %km_homtest(kmdata.test, &testX., &testY., &type., &size., &afont.,
    kmdata.htest);
/* Hazard ratio */
%if "&hr." = "1" %then
  %km_hr(kmdata.hre, kmdata.cs, &hrX., &hrY., &straN., &step., &size.,
    &afont., kmdata.hr);
/* Quantile estimate */
%if "&mst." = "1" %then
  %km_qest(kmdata.qest, strata, &mLabX., &mMedX., &mCIX., &mstY.,
    &straN., &step., &size., &afont., kmdata.qe);

data &out.;
  set &out.;
  rename time = &time.;
run;

%if %eval(&censEXT.+&atrisk.+&test.+&hr.+&ci.+&mst.) ^= 0 %then %do;
  data &anno.;
    length when $1. style $30.;
    set
      %if &ci. = 1 %then kmdata.ci;
      %if &censEXT. = 1 %then kmdata.cens;
      %if &atrisk. = 1 %then kmdata.atrisk;
      %if &test. = 1 %then kmdata.htest;
      %if &hr. = 1 %then kmdata.hr;
      %if &mst. = 1 %then kmdata.qe;
      ;
   retain when 'B';
  run;
%end;

proc datasets lib = kmdata;
  delete cs ple qest test
    %if &adjbase. = 0 %then hre;
    %if &CI. = 1 %then ci;
    %if &censEXT. = 1 %then cens;
    %if &atrisk. = 1 %then atrisk;
    %if &test. = 1 %then htest;
    %if &hr. = 1 %then hr;
    %if &MST. = 1 %then qe;
    ;
run; quit;
libname kmdata CLEAR;

%mend km_data;


/********************************************************/
/* confidence interval */
%macro km_ci(
  confInt, time, straN, pattern, lib, out
);
proc sort data = &confInt.; by time;
%do i = 1 %to &straN.;
  data &lib..tmp&i.;
    length color function $10.;
    retain xsys ysys '2' style "&pattern." line 0 xtmp ytmp .;
    set &confInt.(firstobs = 2);
    where SL&i. ^= .;
    color = "&&scolor&i.";
    function = 'move'; x = xtmp;   y = ytmp;  output;
    function = 'bar';  x = &time.; y = SU&i.; output;
    xtmp = &time.; ytmp = SL&i.;
    keep xsys ysys style line color function x y;
%end;
data &out.;
set
%do i = 1 %to &straN.;
  &lib..tmp&i.(firstobs = 3)
%end;;
%do i = 1 %to &straN.;
  proc datasets lib = &lib.;
  delete tmp&i.;
%end;;
run;
%mend km_ci;



/********************************************************/
/* censor extension */
%macro km_censEXT(
  data, time, straN, cHeight, cWidth, out
);
data &out.;
  length color function $10.; 
  retain xsys ysys '2' size &cWidth.; set &data.;
  %do i = 1 %to &straN.;
    if CSDF&i. ^= . then do;
      color = "&&color&i.";
      function='move'; x=&time.; y=CSDF&i.; output;
      function='draw'; x=&time.; y=CSDF&i. + &cHeight.; output;
    end;
  %end;
  keep xsys ysys size color function x y;
run;
%mend km_censEXT;



/********************************************************/
/* No. at risk */
%macro km_atrisk(
  data, stra, time, straN, order, base, step, size, label,
  afont, atriskYsys, atriskPos, atriskStep, lib, out
);
proc sort data = &data.; by STRATUM &time.;
data &out.;
  set &data.; by STRATUM &time.;
  do x = &order.;
    if &time. = 0 & Censor = . then do; output; leave; end;
    else if &time. <= x then do; output; leave; end;
  end;
data &out.;
  length color function $10.; set &out.; by STRATUM x;
  retain xsys '2' ysys "&atriskYsys." color 'black' position "&atriskPos."
    function 'label' size &size.;
  %do i = &straN. %to 1 %by -1;
    if STRATUM = &i. then y =
      (&base. + &atriskStep. * (&straN. - 1)) - &atriskStep. * (&i. - 1);
  %end;
  text = cat(Left);
  if last.x then output;
data &lib..dummy;
  length color function $10.; 
  retain xsys '2' ysys "&atriskYsys." color 'black' position "&atriskPos."
    function 'label' size &size.;
  do STRATUM = 1 to &straN.;
  do x = &order.;
    %do i = &straN. %to 1 %by -1;
      if STRATUM = &i. then y =
        (&base. + &atriskStep. * (&straN. - 1)) - &atriskStep. * (&i. - 1);
    %end;
    output;
  end; end;
data &out.;
  merge &lib..dummy &out.; by STRATUM x;
  format xtext $20.; retain xtext '';
  if text ^= "" then xtext = text;
  else text = xtext;
data &lib..label;
  length color function $10. text $100.;
  retain xsys '2' ysys "&atriskYsys." color 'black' position "&atriskPos."
    function 'label' size &size.;
  x = 0; y = &base. + &atriskStep. * &straN.; text = &label.; output;
data &out.; set &out. &lib..label; retain style &afont.;
  keep xsys ysys color position function size x y text style;
proc datasets lib = &lib; delete dummy Label;
run;
%mend km_atrisk;



/********************************************************/
/* homogeneity test */
%macro km_homtest(
  Test, testX, testY, type, size, afont, out
);
data &out.;
  %if "&type" = "logrank" %then
    set &Test.(firstobs = 1 obs = 1);
  %else %if "&type" = "wilcoxon" %then
    set &Test.(firstobs = 2 obs = 2);
  %else %if "&type" = "likelihoodratio" %then
    set &Test.(firstobs = 3 obs = 3);;
  retain xsys '1' ysys '1' color 'black' position '7'
    function 'label' size &size. style &afont.;
  x = &testX.; y = &testY.;
  %if "&type" = "logrank" %then
    text = cat("Log-rank P-value = ", put(ProbChiSq, PVALUE5.4));
  %else %if "&type" = "wilcoxon" %then
    text = cat("Wilcoxon P-value = ", put(ProbChiSq, PVALUE5.4));
  %else %if "&type" = "likelihoodratio" %then
    text = cat("Likelihood ratio P-value = ", put(ProbChiSq, PVALUE5.4));;
  keep xsys ysys color position function size style x y text;
run;
%mend km_homtest;



/********************************************************/
/* hazard ratio */
%macro km_hr(
  HazardRatio, classInfo, hrX, hrY, straN, step, size, afont, out
);
proc transpose data = &classInfo. out = &classInfo.;
  var strata;
data &out.;
  set &HazardRatio.;
  if _n_ = 1 then set &classInfo.;
  length text $200.;
  retain xsys '1' ysys '1' color 'black' position '7' function 'label'
      size &size. style &afont.;
  x = &hrX.;
  y = &hrY. - &step. * (_n_ - 1);
  %do i = 2 %to &straN.;
    if _n_ = &i. - 1 then do;
      text = cat(
               compress(COL&i.), " v.s. ",
               compress(COL1), " HR = ", 
               compress(put(HazardRatio, 8.3)), " [", 
               compress(put(hrLowerCL, 8.3)), ", ",
               compress(put(hrUpperCL, 8.3)), "]"
             );
    end;
  %end;
  keep xsys ysys color position function size style x y text;
run;
%mend km_hr;



/********************************************************/
/* Quantile estimate + 95% CI */
%macro km_qest(
  Est, stra, mLabX, mMedX, mCIX, mstY, straN, step,
  size, afont, out
);
data &out.;
  length text $200.;
  set &Est.;
  retain xsys '1' ysys '1' color 'black' function 'label'
      size &size. style &afont.;
  position = '4';
  x = &mMedX.; y = &mstY. - &step.; text = "mst"; output;
  x = &mCIX.;  y = &mstY. - &step.; text = "95% C.I."; output;
  %do i = 1 %to &straN.;
  if (STRATUM = &i. & Percent = 50) then do;
    position = '4';
    y = &mstY. - &step. * (&i. + 1);
    x = &mMedX.; text = put(Estimate, 6.1); output;
    x = &mCIX.;
    text = cat("[", compress(put(LowerLimit, 6.1)),
               ", ", compress(put(UpperLimit, 6.1)), ")"); output;
    position = '6';
    x = &mLabX.; text = trim(&stra.); output;
  end;
  %end;
  keep xsys ysys color position function size style x y text;
run;
%mend km_qest;

